package de.ozgcloud.xta.test.app.config;

import jakarta.xml.ws.Endpoint;
import jakarta.xml.ws.soap.SOAPBinding;

import org.apache.cxf.bus.spring.SpringBus;
import org.apache.cxf.jaxws.EndpointImpl;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import genv3.de.xoev.transport.xta.x211.ManagementPortType;
import genv3.de.xoev.transport.xta.x211.MsgBoxPortType;
import genv3.de.xoev.transport.xta.x211.SendPortType;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Configuration
@Log4j2
@RequiredArgsConstructor
public class XTAServer {

	private static final String PATH_PREFIX = "/XTA210";

	private final SpringBus bus;

	@Bean
	Endpoint managementEndpoint(final ManagementPortType managementPortType) {
		EndpointImpl endpoint = new EndpointImpl(bus, managementPortType);
		endpoint.publish(PATH_PREFIX + "managementPort.svc");
		configureEndpoint(endpoint);
		return endpoint;
	}

	@Bean
	Endpoint msgBoxEndpoint(final MsgBoxPortType msgBoxPortType) {
		EndpointImpl endpoint = new EndpointImpl(bus, msgBoxPortType);
		endpoint.publish(PATH_PREFIX + "msgBoxPort.svc");
		configureEndpoint(endpoint);
		return endpoint;
	}

	@Bean
	Endpoint sendXtaEndpoint(final SendPortType sendPortType) {
		EndpointImpl endpoint = new EndpointImpl(bus, sendPortType);
		endpoint.publish(PATH_PREFIX + "sendPort.svc");
		configureEndpoint(endpoint);
		return endpoint;
	}

	private void configureEndpoint(final EndpointImpl endpoint) {
		log.debug("configuring endpoint {}", endpoint.getBeanName());
		SOAPBinding binding = (SOAPBinding) endpoint.getBinding();
		binding.setMTOMEnabled(true);
	}
}
