package de.ozgcloud.xta.test.app.mapper;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import jakarta.activation.DataHandler;

import org.apache.cxf.ws.addressing.AttributedURIType;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.xta.test.app.exception.TechnicalException;
import de.ozgcloud.xta.test.app.model.*;

import genv3.de.xoev.transport.xta.x211.ContentType;
import genv3.de.xoev.transport.xta.x211.GenericContentContainer;
import genv3.de.xoev.transport.xta.x211.LookupServiceRequest;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxCloseRequestType;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxFetchRequest;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxStatusListRequestType;
import genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import genv3.eu.osci.ws.x2014.x10.transport.PartyIdentifierType;
import genv3.eu.osci.ws.x2014.x10.transport.PartyType;
import genv3.eu.osci.ws.x2014.x10.transport.QualifierType;

@Mapper(
		unmappedTargetPolicy = ReportingPolicy.ERROR
)
public interface RequestMapper {

	@Mapping(target = "metaData", source = "messageMetaData")
	@Mapping(target = "messageFile", source = "genericContainer.contentContainer.message")
	@Mapping(target = "attachmentFiles", source = "genericContainer.contentContainer.attachment")
	XtaMessage mapXtaMessageFromGenericContentContainer(GenericContentContainer genericContainer, MessageMetaData messageMetaData);

	@Mapping(target = "service", source = "qualifier.service")
	@Mapping(target = "businessScenarioCode", source = "qualifier.businessScenario", qualifiedByName = "mapBusinessScenarioCode")
	@Mapping(target = "businessScenarioName", source = "qualifier.businessScenario.defined.name")
	@Mapping(target = "businessScenarioListUri", source = "qualifier.businessScenario.defined.listURI")
	@Mapping(target = "businessScenarioListVersionId", source = "qualifier.businessScenario.defined.listVersionID")
	@Mapping(target = "messageTypeCode", source = "qualifier.messageType.code")
	@Mapping(target = "messageTypePayloadSchema", source = "qualifier.messageType.payloadSchema")
	@Mapping(target = "messageId", source = "msgIdentification.messageID.value")
	@Mapping(target = "authorIdentifier", source = "originators.author.identifier")
	@Mapping(target = "readerIdentifier", source = "destinations.reader.identifier")
	@Mapping(target = "messageSize", source = "msgSize")
	XtaMessageMetaData mapXtaMessageMetaData(MessageMetaData messageMetaData);

	@Named("mapBusinessScenarioCode")
	default String mapBusinessScenarioCode(QualifierType.BusinessScenario businessScenario) {
		return businessScenario.getDefined() != null ? businessScenario.getDefined().getCode() : businessScenario.getUndefined();
	}

	@Mapping(target = "file", source = "value")
	@Mapping(target = "name", source = "filename")
	@Mapping(target = "language", source = "lang")
	XtaFile mapXtaFileFromContentType(ContentType contentType);

	default File persistToTempFile(DataHandler dataHandler) {
		try {
			var file = File.createTempFile("xta", ".data");
			file.deleteOnExit();
			try (var out = new FileOutputStream(file)) {
				dataHandler.writeTo(out);
			}
			return file;
		} catch (IOException e) {
			throw new TechnicalException("Error writing Attachment to temp file", e);
		}
	}

	default String mapMessageIdFromMsgBoxFetchRequest(MsgBoxFetchRequest fetchRequest) {
		return fetchRequest.getMsgSelector().getMessageID().getFirst().getValue();
	}

	@Mapping(target = "clientIdentifier", source = "clientIdentifier.identifier")
	@Mapping(target = "maxListItems", source = "fetchRequest.maxListItems")
	XtaMessageMetaDataListingRequest mapMessageMetaDataListingRequest(MsgBoxStatusListRequestType fetchRequest, PartyType clientIdentifier);

	XtaIdentifier mapXtaIdentifier(PartyIdentifierType partyType);

	@Mapping(target = "messageId", source = "closeRequest.lastMsgReceived.first.value")
	@Mapping(target = "clientIdentifier", source = "authorIdentifier.identifier")
	XtaCloseRequest mapXtaCloseRequest(MsgBoxCloseRequestType closeRequest, PartyType authorIdentifier);

	@Mapping(target = "messageId", source = "fetchRequest.msgSelector.messageID.first.value")
	@Mapping(target = "clientIdentifier", source = "authorIdentifier.identifier")
	XtaGetMessageRequest mapXtaGetMessageRequest(MsgBoxFetchRequest fetchRequest, PartyType authorIdentifier);

	@Mapping(target = "service", source = "lookupServiceRequest.lookupServiceRequestList.first.lookupService.serviceType")
	@Mapping(target = "readerIdentifier", source = "lookupServiceRequest.lookupServiceRequestList.first.lookupService.reader.identifier")
	@Mapping(target = "authorIdentifier", source = "authorIdentifier.identifier")
	XtaLookupServiceRequest mapXtaLookupServiceRequest(LookupServiceRequest lookupServiceRequest, PartyType authorIdentifier);

	@Mapping(target = "messageId", source = "messageId.value")
	@Mapping(target = "clientIdentifier", source = "authorIdentifier.identifier")
	XtaGetTransportReportRequest mapXtaGetTransportReportRequest(AttributedURIType messageId, PartyType authorIdentifier);
}
