package de.ozgcloud.xta.test.app.service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.function.Predicate;

import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import de.ozgcloud.xta.test.app.data.XtaMessageRepository;
import de.ozgcloud.xta.test.app.model.*;

import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class XtaMessageService {
	private final XtaMessageRepository messageRepository;

	private final Environment environment;

	public boolean sendMessage(XtaMessage message) {
		return messageRepository.add(message);
	}

	public XtaMessageMetaDataListing getStatusList(XtaMessageMetaDataListingRequest listingRequest) {
		return XtaMessageMetaDataListing.builder()
				.pendingMessageCount(messageRepository.countOpenMessagesByReaderIdentifierValue(listingRequest.clientIdentifier().value()))
				.messages(getMetaData(messageRepository.findOpenMessagesByReaderIdentifierValueLimitedTo(
						listingRequest.clientIdentifier().value(),
						listingRequest.maxListItems())))
				.build();
	}

	private List<XtaMessageMetaData> getMetaData(List<XtaMessage> messages) {
		return messages.stream()
				.map(XtaMessage::metaData)
				.toList();
	}

	public Optional<XtaMessage> getMessage(XtaGetMessageRequest request) {
		return messageRepository.getMessage(request.messageId())
				.filter(isAuthorOrReaderOfMessage(request.clientIdentifier().value()));
	}

	Predicate<XtaMessage> isAuthorOrReaderOfMessage(String clientIdentifierValue) {
		return m -> isAuthorOrReaderOfMessageMetadata(clientIdentifierValue).test(m.metaData());
	}

	public boolean closeMessage(XtaCloseRequest request) {
		return messageRepository.closeByMessageIdAndReaderIdentifierValue(
				request.messageId(),
				request.clientIdentifier().value()
		);
	}

	public Optional<XtaTransportReport> getTransportReport(XtaGetTransportReportRequest request) {
		return messageRepository.getTransportReport(request.messageId())
				.filter(isAuthorOrReaderOfTransportReport(request.clientIdentifier().value()));
	}

	Predicate<XtaTransportReport> isAuthorOrReaderOfTransportReport(String clientIdentifierValue) {
		return tr -> isAuthorOrReaderOfMessageMetadata(clientIdentifierValue).test(tr.messageMetaData());
	}

	Predicate<XtaMessageMetaData> isAuthorOrReaderOfMessageMetadata(String clientIdentifierValue) {
		return m -> m.readerIdentifier().value().equals(clientIdentifierValue)
				|| m.authorIdentifier().value().equals(clientIdentifierValue);
	}

	public String createMessageId() {
		return generateXtaIdWithNamespace("de:xta:messageid");
	}

	public String createRequestId() {
		return generateXtaIdWithNamespace("de:xta:requestid");
	}

	private String generateXtaIdWithNamespace(String namespace) {
		return String.join(":",
				"urn",
				namespace,
				environment.getProperty("spring.application.name"),
				UUID.randomUUID().toString()
		);
	}
}
