package de.ozgcloud.xta.test.app.validation;

import java.util.Collection;
import java.util.function.Function;
import java.util.stream.Collectors;

import jakarta.annotation.Nullable;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.Validation;
import jakarta.validation.Validator;

import org.springframework.stereotype.Service;

import de.ozgcloud.xta.test.app.mapper.ExceptionMapper;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class RequestValidator {

	private static final Validator VALIDATOR;

	static {
		try (var factory = Validation.buildDefaultValidatorFactory()) {
			VALIDATOR = factory.getValidator();
		}
	}

	private final ExceptionMapper exceptionMapper;

	public <T> void validate(@Nullable T request) throws XTAWSTechnicalProblemException {
		if (request == null) {
			throw exceptionMapper.createTechnicalProblemException("Request is invalid: null");
		}
		validate(request, violations ->
				exceptionMapper.createTechnicalProblemException("Request is invalid:\n" + getViolationsDescription(violations))
		);
	}

	private <T> String getViolationsDescription(Collection<ConstraintViolation<T>> violations) {
		return violations.stream()
				.map(v -> "'%s' %s".formatted(v.getPropertyPath().toString(), v.getMessage()))
				.collect(Collectors.joining("\n"));
	}

	private <T, E extends Exception> void validate(T request, Function<Collection<ConstraintViolation<T>>, E> violationsConsumer) throws E {
		var violations = VALIDATOR.validate(request);
		if (!violations.isEmpty()) {
			throw violationsConsumer.apply(violations);
		}
	}
}
